rem
rem Paint tool
rem

img_w = 12
img_h = 12

sel_tool = -1
sel_color = 8
sel_rot = 0

dim img(img_h, img_w)
dim colors(15)
for i = 0 to ubound(colors)
  pset 0, 0, i
  colors[i] = get_rgb(-point(0, 0))
next

bn_y = 86
rgb_x = 294

const bn_w = 30
const bn_h = 30
const border = 20
const rgb_w = 30
const c_button = rgb(34, 55, 0)
const c_grid = rgb(10, 10, 10)
const c_grid_bg = rgb(200, 200, 200)
const margin = 10
const slider1 = 0
const slider2 = 1
const slider3 = 2

widgets = []
widgets << mk_slider(84, margin, 280, 20, 0, @red)
widgets << mk_slider(84, 35, 280, 45, 1, @green)
widgets << mk_slider(84, 60, 280, 70, 2, @blue)
widgets << mk_grid(60, 80)
widgets << mk_preview(margin, margin, 70, 70)
widgets << mk_button("New", @click_new, "new.xpm")
widgets << mk_button("Load", @click_load, "load.xpm")
widgets << mk_button("Save As", @click_save, "save.xpm")
widgets << mk_button("Rotate", @click_rotate, "rotate.xpm")
widgets << mk_button("Erase", @click_tool, "erase.xpm")
widgets << mk_button("Lighten", @click_tool, "lighten.xpm")
widgets << mk_button("Darken", @click_tool, "darken.xpm")

for i = 0 to ubound(colors)
  widgets << mk_rgb(i)
next

func get_rgb(c)
  r = (c band 0xff0000) rshift 16
  g = (c band 0xff00) rshift 8
  b = c band 0xff
  return [r, g, b]
end

func darken(c, n)
  c[0] = max(1, min(c[0] + n, 255))
  c[1] = max(1, min(c[1] + n, 255))
  c[2] = max(1, min(c[2] + n, 255))
  return -rgb(c[0], c[1], c[2])
end

sub iterate(w, h, filter)
  local x, y
  for y = 0 to h - 1
    for x = 0 to w - 1
      call filter, x, y
    next
  next
end

func scale_image(img, w, h)
  local srcW = img.width
  local srcH = img.height
  local yscale = h / srcH
  local xscale = w / srcW
  local _in, _out

  dim _out(h - 1, w - 1)
  img.save(_in)

  sub filter(x, y)
    local x1 = int(x / xscale)
    local y1 = int(y / yscale)
    _out[y, x] = _in[y1, x1]
  end

  iterate(w, h, @filter)
  return image(_out)
end

func dialog(title, xpm)
  local th = txth("Q")
  local x1 = xmax / 3
  local x2 = x1 * 2
  local y1 = ymax / 3
  local y2 = y1 + th * 8
  local tx = x1 + 10

  cls
  rect x1, y1, x2, y2, 2
  rect x1 + 2, y1 + 2, x2 - 2, y2 - 2, 2
  at tx, y1 + (th * 1): print "=== " + title + " ==="
  at tx, y1 + (th * 3): print "Press enter to cancel"
  at tx, y1 + (th * 5): input "Enter file name: ", k

  if (xpm and k != 0 and "xpm" != rightoflast(k, ".")) then
    k += ".xpm"
  endif
  return k
end

func red(vr)
  return rgb(160 - 5 * vr, 0, 0)
end

func green(vr)
  return rgb(0, 160 - 5 * vr, 0)
end

func blue(vr)
  return rgb(0, 0, 160 - 5 * vr)
end

func mk_rect(x1, y1, x2, y2)
  local result = {}
  result.x1 = x1
  result.y1 = y1
  result.x2 = x2
  result.y2 = y2
  return result
end

func mk_grid(x1, y1)
  local x2 = xmax - border
  local y2 = ymax - border
  local result = mk_rect(x1, y1, x2, y2)
  result.draw_fn = @draw_grid
  result.click_fn = @click_grid
  return result
end

func mk_preview(x1, y1, x2, y2)
  local result = mk_rect(x1, y1, x2, y2)
  result.draw_fn = @draw_preview
  result.click_fn = @click_null
  return result
end

func mk_rgb(index)
  local x1 = rgb_x
  local y1 = margin
  local result = mk_rect(x1, y1, x1 + rgb_w, y1 + 60)
  rgb_x += rgb_w + 3
  result.index = index
  result.draw_fn = @draw_rgb
  result.click_fn = @click_rgb
  return result
end

func mk_slider(x1, y1, x2, y2, i, col_fn)
  local result = mk_rect(x1, y1, x2, y2)
  result.index = i
  result.pos = (x2 - x1) / 2
  result.col_fn = col_fn
  result.draw_fn = @draw_slider
  result.click_fn = @click_slider
  return result
end

func mk_button(title, click_fn, file)
  local x1 = margin + 4
  local y1 = bn_y
  local result = mk_rect(x1, y1, x1 + bn_w, y1 + bn_h)
  local img_data

  bn_y += bn_h + txth("q") + 15
  if (exist(file)) then
    tload file, img_data
    result.img = scale_image(image(img_data), bn_w, bn_h)
  else
    result.img = 0
  endif

  result.draw_fn = @draw_button
  result.click_fn = @click_fn
  result.title = title
  result.title_offs = (bn_w - txtw(title)) / 2
  return result
end

sub click_new(x, y, byref w)
  local w = window()
  local k
  w.ask("Are you sure?", "New Image")
  if (w.answer == 0) then
    cls
    input "Enter width: ", k: img_w = max(5, int(k))
    input "Enter height: ", k: img_h = max(5, int(k))
    dim img(img_h, img_w)
  endif
end

sub click_tool(x, y, byref w)
  sel_tool = w.title
end

sub click_grid(x, y, byref grid)
  local ax = int((x - grid.x1) / grid.cell_w)
  local ay = int((y - grid.y1) / grid.cell_h)
  local col = colors[sel_color]
  if (ax >= 0 and ay >= 0) then
    if (sel_tool == "Erase") then
      img[ay, ax] = 0
    else if (sel_tool == "Lighten") then
      if (img[ay, ax] != 0 and (last_ax != ax or last_ay != ay)) then 
        img[ay, ax] = darken(get_rgb(img[ay, ax]), 10)
        last_ax = ax
        last_ay = ay
      endif
    else if (sel_tool == "Darken") then
      if (img[ay, ax] != 0 and (last_ax != ax or last_ay != ay)) then 
        img[ay, ax] = darken(get_rgb(img[ay, ax]), -10)
        last_ax = ax
        last_ay = ay
      endif
    else
      img[ay, ax] = -rgb(col[0], col[1], col[2])
      if (img[ay, ax] == 0) then img[ay, ax] = -rgb(1, 0, 0)
    endif
  endif
end

sub click_load(x, y, byref w)
  local k = dialog("Load file", false)
  local xpm_txt, i, y, x, r, g, b
  if (exist(k)) then
    if ("xpm" == rightoflast(k, ".")) then
      tload k, xpm_txt
      i = scale_image(image(xpm_txt), img_w, img_h)
    else
      i = scale_image(image(k), img_w, img_h)
    endif
    i.save(img)
    sub filter(x1, y1)
      [r, g, b] = get_rgb(img[y1, x1])
      img[y1, x1] = -rgb(r, g, b)
    end
    iterate(img_w, img_h, @filter)
  else if (len(k) > 0) then
    cls: print "File not found: " + k
    showpage
    pause
  endif
end

sub click_null(x, y, byref w)
end

sub click_rgb(x, y, byref w)
  sel_color = w.index

  local width = widgets[slider1].x2 - widgets[slider1].x1
  local r = colors[sel_color][0]
  local g = colors[sel_color][1]
  local b = colors[sel_color][2]

  widgets[slider1].pos = r * width / 255
  widgets[slider2].pos = g * width / 255
  widgets[slider3].pos = b * width / 255
  sel_tool = -1
end

sub click_rotate(x, y, byref w)
  local out
  dim out(img_h, img_w)

  sel_rot = iff(sel_rot, false, true)

  sub filter(x1, y1)
    local x2, y2
    if (sel_rot) then
      x2 = img_w - x1 - 1
      y2 = y1
    else
      x2 = x1
      y2 = img_h - y1 - 1
    endif
    out[y1, x1] = img[y2, x2]
  end
  iterate(img_w, img_h, @filter)
  img = out
end

sub click_save(x, y, byref w)
  func save_xpm
    local x, y, c
    local map = {}
    local out = ""
    local c1 = 0
    local c2 = 0
    local first_ch = 33
    local ch = chr(first_ch + c1) + chr(first_ch + c2)
    local row
    for y = 0 to img_h - 1
      for x = 0 to img_w - 1
        map[img[y, x]]++
      next
    next
    out += img_w + " " + img_w + " " + len(map) + " 2\n"
    for c in map
      map[c] = ch
      out += ch + " c " + iff(c > 0, ("#" + hex(c)), "none") + "\n"
      if (c2 == 94) then
        c1++
        c2 = 0
      else
        c2++
      endif
      ch = chr(first_ch + c1) + chr(first_ch + c2)
    next
    for y = 0 to img_h - 1
      row = ""
      for x = 0 to img_w - 1
        row += map[img[y, x]]
      next
      out += row + "\n"
    next
    return out
  end

  local k = dialog("Save file", true)
  if (len(k) > 0) then
    local xpm_txt = save_xpm
    tsave k, xpm_txt
  endif
end

sub click_slider(x, y, byref slider)
  local width = slider.x2 - slider.x1
  slider.pos = min(width, max(0, x - slider.x1))
  colors[sel_color][slider.index] = slider.pos * 255 / width
end

sub draw_button(byref w, focus)
  local x1 = w.x1
  if (sel_tool != 0 and sel_tool == w.title) then
    rect x1 - 2, w.y1 - 2, w.x2 + 2, w.y2 + 2, c_button
  endif
  if (ismap(w.img)) then
    w.img.draw(x1, w.y1)
  else
    rect x1, w.y1, w.x2, w.y2, c_button
  endif
  at w.x1 + w.title_offs, w.y2 + 4
  print w.title
end

sub draw_grid(byref w, focus)
  local x, y, x1, x2, y1, y2

  w.x2 = xmax - border
  w.y2 = ymax - border
  w.cell_w = int((w.x2 - w.x1) / img_w)
  w.cell_h = int((w.y2 - w.y1) / img_h)

  for y = 0 to img_h - 1
    for x = 0 to img_w - 1
      x1 = w.x1 + (x * w.cell_w)
      x2 = x1 + w.cell_w
      y1 = w.y1 + (y * w.cell_h)
      y2 = y1 + w.cell_h
      if (img[y, x] != 0) then
        rect x1, y1, x2, y2, -img[y, x] filled
      else
        rect x1 + 1, y1 + 1, x2 - 1, y2 - 1, c_grid_bg filled
      endif
    next
  next

  rect w.x1 - 1, w.y1 - 1, x2, y2, c_grid
end

sub draw_preview(byref pv, focus)
  local w = pv.x2 - pv.x1
  local h = pv.y2 - pv.y1
  local yscale = h / img_h
  local xscale = w / img_w
  local out

  dim out(h - 1, w - 1)

  sub filter(x, y)
    local x1 = int(x / xscale)
    local y1 = int(y / yscale)
    local c = img[y1, x1]
    local r, g, b
    local r = (c band 0xff0000) rshift 16
    local g = (c band 0xff00) rshift 8
    local b = c band 0xff
    if (r == 0 and g == 0 and b == 0) then
      r = 255
      g = 255
      b = 255
    endif
    local px = (255 lshift 24) + (r lshift 16) + (g lshift 8) + b
    out[y, x] = px
  end

  iterate(w, h, @filter)
  local i = image(out)
  i.draw(pv.x1, pv.y1)
end

sub draw_rgb(byref w, focus)
  local col = colors[w.index]
  rect w.x1, w.y1, w.x2, w.y2, rgb(col[0], col[1], col[2]) filled
  if (w.index == sel_color and sel_tool == -1) then
    rect w.x1, w.y1, w.x2, w.y2, 0
  endif
end

sub draw_slider(byref slider, focus)
  local r, vr
  local size = (slider.y2 - slider.y1)
  local nstep = size / 10
  local x = slider.x1 + slider.pos
  local y = slider.y1 + (slider.y2 - slider.y1) / 2
  rect slider.x1, slider.y1, slider.x1 + slider.pos, slider.y2, slider.col_fn(0) filled
  rect slider.x1 + slider.pos, slider.y1 + 1, slider.x2, slider.y2 - 2, 7 filled
  for r = size to 1 step - nstep
    vr = r * 25 / size
    circle x, y, r, 1, slider.col_fn(vr) filled
  next
end

sub draw_widgets(focus)
  local i
  color 1, rgb(131, 141, 151)
  cls
  for i = 0 to len(widgets) - 1
    call widgets[i].draw_fn, widgets[i], i == focus
  next
  showpage
end

func in_rect(x, y, rc)
  return iff(x < rc.x1 OR x > rc.x2 OR y < rc.y1 OR y > rc.y2, false, true)
end

sub main
  local x, y, i, k
  local focus = -1
  local width = xmax
  local height = ymax

  draw_widgets(focus)

  while 1
    while !pen(0)
      k = inkey
      if (asc(k) == 9) then
        sel_color = (sel_color + 1) % (ubound(colors) + 1)
      endif
      focus = -1
      if (width != xmax or height != ymax or asc(k) == 9) then
        width = xmax
        height = ymax
        draw_widgets(focus)
      endif
    wend

    x = pen(4)
    y = pen(5)

    if (focus == -1) then
      for i = 0 to len(widgets) - 1
        if (in_rect(x, y, widgets[i])) then
          focus = i
          exit for
        endif
      next
    endif

    if (focus != -1) then
      call widgets[i].click_fn, x, y, widgets[i]
      draw_widgets(focus)
    endif
  wend
end

main
