REM /*******************************************************************************************
REM *
REM *   Physac - Body shatter
REM *
REM *   Copyright (c) 2016-2018 Victor Fisac
REM *
REM ********************************************************************************************/

import raylib as rl
import raylibc as c

const screenWidth = 800
const screenHeight = 450

rl.SetConfigFlags(FLAG_MSAA_4X_HINT)
rl.InitWindow(screenWidth, screenHeight, "SmallBASIC Physac [raylib] - Body shatter")

' Physac logo drawing position
logoX = screenWidth - rl.MeasureText("Physac", 30) - 10
logoY = 15
needsReset = false

' Initialize physics and default physics bodies
rl.InitPhysics()
rl.SetPhysicsGravity(0, 0)

' Create random polygon physics body to shatter
_ = rl.CreatePhysicsBodyPolygon([screenWidth/2, screenHeight/2], rl.GetRandomValue(80, 200), rl.GetRandomValue(3, 8), 10)
rl.SetTargetFPS(60)               ' Set our game to run at 60 frames-per-second

while (!rl.WindowShouldClose())
  rl.UpdatePhysics()
  if (needsReset) then
    ' Create random polygon physics body to shatter
    _ = rl.CreatePhysicsBodyPolygon([screenWidth/2, screenHeight/2], rl.GetRandomValue(80, 200), rl.GetRandomValue(3, 8), 10)
    needsReset = false
  endif

  if (rl.IsKeyPressed(asc("R"))) then
    rl.ResetPhysics()
    needsReset = true
  endif

  if (rl.IsMouseButtonPressed(c.MOUSE_LEFT_BUTTON)) then   ' Physics shatter input
    ' Note: some values need to be stored in variables due to asynchronous changes during main thread
    count = rl.GetPhysicsBodiesCount()
    for i = count - 1 to 0 step -1
      currentBody = rl.GetPhysicsBody(i)
      if (ismap(currentBody)) then
        rl.PhysicsShatter(currentBody, rl.GetMousePosition(), 10/ currentBody.inverseMass)
      endif
    next i
  endif
    
  rl.BeginDrawing()
  rl.ClearBackground(c.BLACK)
  
  ' Draw created physics bodies
  bodiesCount = rl.GetPhysicsBodiesCount()
  for i = 0 to bodiesCount - 1
    body = rl.GetPhysicsBody(i)
    if (ismap(body)) then
      vertexCount = rl.GetPhysicsShapeVerticesCount(i)
      for j = 0 to vertexCount -1
        ' Get physics bodies shape vertices to draw lines
        ' Note: GetPhysicsShapeVertex() already calculates rotation transformations
        vertexA = rl.GetPhysicsShapeVertex(body, j)
        jj = iff(j + 1 < vertexCount, j + 1, 0)    ' Get next vertex or first to close the shape
        vertexB = rl.GetPhysicsShapeVertex(body, jj)
        rl.DrawLineV(vertexA, vertexB, c.GREEN)     ' Draw a line between two vertex positions
      next j
    endif
  next i
  
  rl.DrawText("Left mouse button in polygon area to shatter body\nPress 'R' to reset example", 10, 10, 10, WHITE)
  rl.DrawText("Physac", logoX, logoY, 30, WHITE)
  rl.DrawText("Powered by", logoX + 50, logoY - 7, 10, WHITE)
  rl.EndDrawing()
wend

rl.ClosePhysics()       ' Unitialize physics
rl.CloseWindow()        ' Close window and OpenGL context
